using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class TranslateParentBehavior : ItemInputBehavior
    {
        protected Vector2f m_positionVec;

        public TranslateParentBehavior(Item target) : base(target) 
        {
            m_positionVec = Vector2f.ZERO;
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }
        
        //! return value in [0,1] to indicate desire for capturing stroke
        override public float GetCaptureScore(Stroke s) {
            if (TargetItem.Bounds.Contains(s[0]))
                return 1.0f;
            return 0.0f;
        }

        //! return true to continue capture
        override public bool OnCaptureStrokeStart(Stroke s) {
            m_positionVec = TargetItem.Parent.Bounds.Min - s[0];
            return true;
        }

        //! return true to continue capture, false to stop capture
        override public bool OnCapturedStrokeUpdate(Stroke s) {
            if (s.Vertices.Count > 1) {
                Box2f parentBounds = TargetItem.Parent.Bounds;
                parentBounds.MoveMin(s.Last + m_positionVec);
                TargetItem.ParentContainer.Resize(parentBounds);
            }
            return true;
        }

    }
}
