using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class VerticalDragTranslateBehavior : ItemInputBehavior
    {
        public VerticalDragTranslateBehavior(Container target)
            : base(target) 
        {
        }

        Container TargetC {
            get { return (Container)Target; }
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.MayCapture; }
        }

        //! return value in [0,1] to indicate desire for capturing stroke
        override public float GetCaptureScore(Stroke s) {
            if (s.TotalLength > 40                      // wait until stroke is non-zero length
                && s.TotalLength < 400                     // ignore strokes once they get too long
                && s.WeightedAvgWindingAngle < 5           // stroke needs to be basically straight
                && TargetC.ChildAreaBounds.Contains(s.Vertices[0])  // stroke must start in childarea
                && Math.Abs(s.EndsDirection.Dot(Vector2f.UP)) > 0.97)
                return Math.Abs(s.EndsDirection.Dot(Vector2f.UP));
            return 0.0f;
        }

        //! return true to continue capture
        override public bool OnCaptureStrokeStart(Stroke s) {
            return true;
        }

        //! return true to continue capture, false to stop capture
        override public bool OnCapturedStrokeUpdate(Stroke s) {
            int nVerts = s.Vertices.Count;
            if (nVerts > 1) {
                Vector2f vDelta = s.LastDelta;

                vDelta.X = 0;
                vDelta.Y = -vDelta.Y;
                TargetC.ScrollChildArea(vDelta, true);
            }
            return true;
        }

    }
}
