using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {

    public class UndoStack {

        public interface UndoableAction {
            bool Undo();
            bool Redo();
        }

        List<UndoableAction> m_vStack;
        int m_nCurrentStackIndex;

        public UndoStack() {
            m_vStack = new List<UndoableAction>();
            m_nCurrentStackIndex = -1;
        }

        public void PushAction(UndoableAction action) {
            Truncate();
            m_vStack.Add(action);
            m_nCurrentStackIndex = m_vStack.Count-1;
        }

        public void Truncate() {
            for ( int i = m_vStack.Count-1; i > m_nCurrentStackIndex; --i )
                m_vStack.RemoveAt(i);
        }


        public void Undo() {
            if (m_vStack.Count == 0 || m_nCurrentStackIndex < 0)
                return;

            if (!m_vStack[m_nCurrentStackIndex].Undo())
                System.Diagnostics.Debugger.Break();
            m_nCurrentStackIndex = m_nCurrentStackIndex-1;
        }

        public void Redo() {
            if (m_vStack.Count == 0 || m_nCurrentStackIndex == m_vStack.Count-1)
                return;
            m_nCurrentStackIndex++;
            if (!m_vStack[m_nCurrentStackIndex].Redo())
                System.Diagnostics.Debugger.Break();
        }


    }
}
