using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    /*
     * tracks all widgets
     * 
     * [TODO] be able to transfer widgets betwen elements?  (need to listen for parent changes...)
     * 
     */

    class WidgetManager
    {
        Dictionary<Element, WidgetSet> m_vWidgets;

        //! widgets which haven't been registered yet (ie inserted into m_vWidgets). If
        //  a widget has no parent, it will stay in this list (should we render these??)
        List<Widget> m_vFreeWidgets;

        Element.ElementChangedHandler m_widgetChangedHandler;


        protected WidgetManager() {
            m_vWidgets = new Dictionary<Element, WidgetSet>();
            m_vFreeWidgets = new List<Widget>();

            m_widgetChangedHandler = new Element.ElementChangedHandler(OnWidgetChanged);
        }
        static WidgetManager s_singleton = null;
        static public WidgetManager Get {
            get { 
                if (s_singleton == null)
                    s_singleton = new WidgetManager();
                return s_singleton;
            }
        }

        //! de-register all widgets connected to Element and clear Parent associations
        public void DisconnectWidgets( Element elem ) {
            if (m_vWidgets.ContainsKey( elem )) {
                WidgetSet widgets = m_vWidgets[elem];
                while (widgets.Count > 0) {
                    Widget w = widgets[0];
                    DeRegister( w );
                    w.Disconnect();
                }
            }
        }


        public void Render() {
            UpdateWidgetAssociations();

            foreach (Element elem in m_vWidgets.Keys) {

                WidgetSet vWidgets = m_vWidgets[elem];
                for ( int i = 0; i < vWidgets.Count; ++i ) {
                    Widget w = vWidgets[i];
                    if (w.Visibility == VisibilityState.Hidden)
                        continue;

                    if (!w.ClipRegion.Intersects( w.Bounds ))
                        continue;

                    Box2f clip = w.ClipRegion.Intersect( w.Bounds );
                    RenderEngine.Get.PushClipBox( clip );
                    w.RenderBackground();
                    RenderEngine.Get.PopClipBox();
                }

                for (int i = 0; i < vWidgets.Count; ++i) {
                    Widget w = vWidgets[i];
                    if (w.Visibility == VisibilityState.Hidden)
                        continue;
                    if (!w.ClipRegion.Intersects( w.Bounds ))
                        continue;

                    Box2f clip = w.ClipRegion.Intersect( w.Bounds );
                    RenderEngine.Get.PushClipBox( clip );
                    w.Render();
                    RenderEngine.Get.PopClipBox();
                }

            }
        }


        public void Register(Widget w) {
            if (!m_vFreeWidgets.Contains(w)) {
                m_vFreeWidgets.Add(w);
                w.ElementChangedEvent += m_widgetChangedHandler;
            }
        }
        public void DeRegister(Widget w) {
            Element e = w.Parent;
            if (e != null && m_vWidgets.ContainsKey( e ) && m_vWidgets[e].Contains( w )) {
                m_vWidgets[e].Remove( w );
                if (m_vWidgets[e].Count == 0)
                    m_vWidgets.Remove( e );
            }
            m_vFreeWidgets.Remove( w );

            w.ElementChangedEvent -= m_widgetChangedHandler;
        }

        //! associate any free widgets with their owner Elements
        void UpdateWidgetAssociations() {
            List<Widget> vAssociated = new List<Widget>();
            foreach (Widget w in m_vFreeWidgets) {
                Element e = w.Parent;
                if (e == null)
                    continue;
                vAssociated.Add( w );
                if (!m_vWidgets.ContainsKey(e))
                    m_vWidgets[e] = new WidgetSet();
                if (!m_vWidgets[e].Contains(w))
                    m_vWidgets[e].Add(w);
            }
            foreach (Widget w in vAssociated)
                m_vFreeWidgets.Remove( w );
        }



        /*
         *  widget redraw events
         */
        public delegate void WidgetRedrawHandler(Widget w);
        public event WidgetRedrawHandler WidgetRedrawEvent;
        protected void NotifyWidgetRedrawEvent(Widget w) {
            if (WidgetRedrawEvent != null)
                WidgetRedrawEvent(w);
        }

        void OnWidgetChanged(object o) {
            Widget w = o as Widget;
            if ( w != null ) 
                NotifyWidgetRedrawEvent( w );
        }

    }
}
