using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    abstract class LayoutHint
    {
    };

    abstract class Layout
    {
        protected bool m_bLayoutValid;

        public Layout() {
            m_bLayoutValid = false;
        }

        public bool Valid {
            get { return m_bLayoutValid; }
        }

        virtual public void InvalidateLayout() {
            m_bLayoutValid = false;
        }

        public Box2f Bounds(Container c) { 
            ValidateLayout(c); 
            return ItemBounds;
        }

        public void ValidateLayout(Container c) {
            if (m_bLayoutValid)
                return;

            // validate layout for this container
            LayoutObjects(c);

            // now validate all child layouts
            for ( int i = 0; i < c.Items.ContainerCount; ++i ) {
                Container childc = c.Items.GetContainer(i);
                childc.ValidateLayout();
            }

            m_bLayoutValid = true;
        }

#region  Layout interface
        protected abstract void LayoutObjects(Container c);

        //! returns computed bounding box of laid-out objects
        protected abstract Box2f ItemBounds {
            get;
        }

        //! support for arbitrary layout hints - default is to ignore
        public virtual void AppendHint(LayoutHint hint) {}
#endregion





#region Layout constants
        public static readonly Box2f MAX_WIDTH_HEIGHT = new Box2f(float.MaxValue, float.MaxValue);
#endregion
    }
}
