using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher.CrossKeys
{
    class CrossKeyboard : InfiniteRectContainer
    {
        List<CrossKeyDisc> m_keys;

        public CrossKeyboard(Box2f bounds)  : base(bounds, new VerticalTilingLayout()) 
        {
            m_keys = new List<CrossKeyDisc>();
            Initialize_8Arc();
        }


        void Initialize_8Arc() {
            int nDiscs = 4;

            float fAngle = (float)(Math.PI/2)  /  (float)nDiscs;
            float fWidth = Math.Min(Bounds.Width, Bounds.Height);

            // analytic formula for determining size of disc (based on chord formula)
            float fRad = (0.5f * fWidth) / (float)(2 * Math.Sin(fAngle/2) + 0.5 * Math.Sin(fAngle));
            float fDiscRad = fRad * (float)Math.Sin( fAngle / 2 );

            // make room for dropshadow and recompute
            float fDropShadowWidth = fDiscRad * 0.3f;
            fWidth -= 2.0f * fDropShadowWidth;
            fRad = (0.5f * fWidth) / (float)(2 * Math.Sin(fAngle/2) + 0.5 * Math.Sin(fAngle));
            fDiscRad = fRad * (float)Math.Sin( fAngle / 2 );
            fDiscRad *= 0.97f;  // fudge a little bit to leave room between discs
            
            // bump angle up a little bit
            float fAngleFudge = fAngle / 10.0f;

            for ( int i = 0; i < nDiscs; ++i ) {
                float fAngleA = (float)i * fAngle + fAngleFudge;
                float fAngleB = (float)(i+1) * fAngle + fAngleFudge;

                Vector2f vA = new Vector2f( fRad * (float)Math.Cos(fAngleA), fRad * (float)Math.Sin(fAngleA) );
                Vector2f vB = new Vector2f( fRad * (float)Math.Cos(fAngleB), fRad * (float)Math.Sin(fAngleB) );
                Vector2f vCenter = new Vector2f( 0.5f * (vA + vB) );
                vCenter.X = Bounds.Width - vCenter.X;

                CrossKeyDisc disc = new CrossKeyDisc(this, fDiscRad);
                disc.Center = vCenter;
                disc.Visibility = VisibilityState.Visible;
                disc.DropShadowWidth = fDropShadowWidth;
                m_keys.Add(disc);
            }
        }
    }
}
