using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class Circle2f : IRegion
    {
        Vector2f m_vCenter;
        float m_fRadius;
        Box2f m_vBounds;

        public Circle2f(Vector2f vCenter, float fRadius) {
            m_vCenter = vCenter;
            m_fRadius = fRadius;
            UpdateBounds();
        }

        public Vector2f Center {
            get { return m_vCenter; }
            set { m_vCenter = value; UpdateBounds();  }
        }

        public float Radius {
            get { return m_fRadius; }
            set { m_fRadius = value; UpdateBounds();  }
        }

        public Box2f Bounds {
            get { return m_vBounds; }
        }


        public bool ClipSegment(Vector2f vStart, Vector2f vEnd,
                                ref Vector2f vClippedStart, ref Vector2f vClippedEnd) {

            Vector2f origin = vStart;
            Vector2f direction = vEnd - vStart;
            float tmax = direction.Normalize();
            if (tmax == 0)
                return false;

            // Calculates a vector from the ray origin to the sphere center.
            Vector2f diff = m_vCenter - origin;
            // Compute the projection of diff onto the ray direction
            float d = diff.Dot(direction);

            float diffSquared = diff.SquaredLength;
            float radiusSquared = m_fRadius * m_fRadius;

            // First rejection test : 
            // if d<0 and the ray origin is outside the sphere than the sphere is behind the ray
            if ((d < 0.0f) && (diffSquared > radiusSquared)) {
                return false;
            }

            // Compute the distance from the sphere center to the projection
            float mSquared = diffSquared - d * d;

            // Second rejection test:
            // if mSquared > radiusSquared than the ray misses the sphere
            if (mSquared > radiusSquared) {
                return false;
            }

            float q = (float)System.Math.Sqrt(radiusSquared - mSquared);

            float t1 = d - q;
            float t2 = d + q;
            if (tmax < t1)
                return false;

            float fHitIntervalLow = Math.Max(0, t1);
            float fHitIntervalHigh = Math.Min(tmax, t2);
            //System.Diagnostics.Debug.WriteLine("Low: " + fHitIntervalLow + "  High: " + fHitIntervalHigh );

            vClippedStart = vStart + fHitIntervalLow * direction;
            vClippedEnd = vStart + fHitIntervalHigh * direction;
            return true;
        }


        void UpdateBounds() {
            m_vBounds.Min = m_vCenter - m_fRadius;
            m_vBounds.Max = m_vCenter + m_fRadius;
        }



#region IRegion Members

        public bool IsInside( Vector2f v ) {
            float fDist = (v - m_vCenter).SquaredLength;
            return fDist < m_fRadius*m_fRadius;
        }

#endregion
    }
}
