using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    public struct ColorRGBA
    {
        public float R;
        public float G;
        public float B;
        public float A;

        public ColorRGBA(float r, float g, float b, float a) {
            R = r; G = g; B = b; A = a;
        }
        public ColorRGBA(float r, float g, float b) {
            R = r; G = g; B = b; A = 1;
        }

        public float ToGrayscaleF() {
            return 0.3f * R + +0.59f * G + 0.11f * B;
        }
        public ColorRGBA ToGrayscale() {
            float f = ToGrayscaleF();
            return new ColorRGBA(f,f,f, A);
        }

        public static ColorRGBA ToRGBA(float r, float g, float b) {
            return new ColorRGBA(r, g, b, 1);
        }
        public static ColorRGBA ToRGBA(float r, float g, float b, float a) {
            return new ColorRGBA(r, g, b, a);
        }
        public static ColorRGBA ToRGBA(double r, double g, double b) {
            return new ColorRGBA((float)r, (float)g, (float)b, 1);
        }
        public static ColorRGBA ToRGBA(double r, double g, double b, double a) {
            return new ColorRGBA((float)r, (float)g, (float)b, (float)a);
        }

        public static ColorRGBA Interpolate( ColorRGBA c1, ColorRGBA c2, float alpha ) {
            return new ColorRGBA(
                (1-alpha) * c1.R + (alpha) * c2.R,
                (1-alpha) * c1.G + (alpha) * c2.G,
                (1-alpha) * c1.B + (alpha) * c2.B,
                (1-alpha) * c1.A + (alpha) * c2.A );
        }


        public static ColorRGBA BLACK = new ColorRGBA(0, 0, 0, 1);
        public static ColorRGBA WHITE = new ColorRGBA(1, 1, 1, 1);

        public static ColorRGBA LIGHTLIGHTGREY = new ColorRGBA(0.9f, 0.9f, 0.9f, 1);
        public static ColorRGBA LIGHTGREY = new ColorRGBA(0.75f, 0.75f, 0.75f, 1);
        public static ColorRGBA GREY = new ColorRGBA(0.5f, 0.5f, 0.5f, 1);
        public static ColorRGBA DARKGREY = new ColorRGBA(0.25f, 0.25f, 0.25f, 1);
        public static ColorRGBA DARKDARKGREY = new ColorRGBA(0.1f, 0.1f, 0.1f, 1);


        public static ColorRGBA RED = new ColorRGBA(1, 0, 0, 1);
        public static ColorRGBA GREEN = new ColorRGBA(0, 1, 0, 1);
        public static ColorRGBA BLUE = new ColorRGBA(0, 0, 1, 1);
        public static ColorRGBA YELLOW = new ColorRGBA(1, 1, 0, 1);
        public static ColorRGBA CYAN = new ColorRGBA(0, 1, 1, 1);
        public static ColorRGBA MAGENTA = new ColorRGBA(1, 0, 1, 1);



    }
}
