using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Sketcher
{
    /*
     * Fast 'unsafe' access to Bitmap pixels. Call LockBitmap() to enable fast access, UnlockBitmap() 
     * when finished. Currently supports RGB and RGBA bitmaps.  (RGB is actually untested, but it should work...)
     * code adapted from post by 'microchip' here: http://www.dreamincode.net/forums/showtopic14788.htm
     * see http://msdn.microsoft.com/library/default....arp11152001.asp
     */
    public unsafe class UnsafeBitmap
    {
        public struct PixelData
        {
            public byte blue;
            public byte green;
            public byte red;
            public byte alpha;
        }

        Bitmap bitmap;
        bool m_bOwnBitmap;
        PixelFormat m_pixelFormat;
        int m_nPixelSize;

        // three elements used for MakeGreyUnsafe
        int nRowWidth;
        BitmapData bitmapData = null;
        Byte* pBase = null;

        public UnsafeBitmap(Bitmap bitmap, bool bOwnBitmap) {
            this.bitmap = new Bitmap(bitmap);
            m_pixelFormat = bitmap.PixelFormat;
            m_nPixelSize = GetPixelSize(m_pixelFormat);
            m_bOwnBitmap = bOwnBitmap;
        }

        public UnsafeBitmap(int width, int height, PixelFormat format) {
            m_pixelFormat = format;
            m_nPixelSize = GetPixelSize(m_pixelFormat);
            this.bitmap = new Bitmap(width, height, m_pixelFormat);
            m_bOwnBitmap = true;
        }

        public bool OwnsBitmap {
            get { return m_bOwnBitmap; }
            set { m_bOwnBitmap = value; }
        }

        public void Dispose() {
            if ( m_bOwnBitmap )
                bitmap.Dispose();
        }

        public Bitmap Bitmap {
            get { return (bitmap); }
        }

        public void LockBitmap() {
            //GraphicsUnit unit = GraphicsUnit.Pixel;
            //RectangleF boundsF = bitmap.GetBounds(ref unit);
            //Rectangle bounds = new Rectangle((int)boundsF.X, (int)boundsF.Y, 
            //                                 (int)boundsF.Width, (int)boundsF.Height);
            Rectangle bounds = new Rectangle(0, 0, bitmap.Width, bitmap.Height);

            // Figure out the number of bytes in a row. Rounded up because a scan
            // line in an image must always be a multiple of 4 bytes in length.
            //nRowWidth = (int)boundsF.Width * m_nPixelSize;
            nRowWidth = bounds.Width * m_nPixelSize;
            if (nRowWidth % 4 != 0) {
                nRowWidth = 4 * (nRowWidth / 4 + 1);
            }
            bitmapData =
                bitmap.LockBits(bounds, ImageLockMode.ReadWrite, m_pixelFormat);

            pBase = (Byte*)bitmapData.Scan0.ToPointer();
        }


        public PixelData GetPixel(int x, int y) {
            PixelData returnValue = *PixelAt(x, y);
            if ( m_nPixelSize < 4 )
                returnValue.alpha = 255;
            return returnValue;
        }

        public void SetPixel(int x, int y, PixelData colour) {
            PixelData* pixel = PixelAt(x, y);
            if ( m_nPixelSize == 3 ) {
                pixel->blue = colour.blue;  pixel->green = colour.green;  pixel->red = colour.red;
            } else
                *pixel = colour;
        }

        public void UnlockBitmap() {
            bitmap.UnlockBits(bitmapData);
            bitmapData = null;
            pBase = null;
        }
        public PixelData* PixelAt(int x, int y) {
            return (PixelData*)(pBase + y * nRowWidth + x * m_nPixelSize);
        }


        public static int GetPixelSize(PixelFormat format) {
            // RMS TODO: this is not finished...
            if ( format == PixelFormat.Format24bppRgb )
                return 3 * sizeof(byte);
            else if ( format == PixelFormat.Format32bppArgb )
                return 4 * sizeof(byte);
            else
                return 3 * sizeof(byte);
        }
    }

}
