using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class AnimationManager
    {
        class RunningAnim {
            public double startTime;
            public double lastTime;
            public Animation anim;
        };
        List<RunningAnim> m_playing;
        List<RunningAnim> m_finished;

        SysTimer m_timer;


        public void AnimateOnce( Animation animation ) {

            // [RMS HACK] timer should already be started...
            if ( ! m_timer.Running )
                m_timer.Start();

            // start animation
            RunningAnim running = new RunningAnim();
            running.anim = animation;
            running.startTime = m_timer.Current;
            running.lastTime = running.startTime;

            m_playing.Add(running);

            animation.Start();
        }


        public void StopAnimation( Animation animation ) {
            foreach ( RunningAnim anim in m_playing ) {
                if ( anim.anim == animation ) {
                    anim.anim.Stop();
                    m_playing.Remove(anim);
                    return;
                }
            }

            // [RMS TODO] should refresh here, no?
        }

        //! return true if anything was animated
        public bool AnimationTick() {

            if ( m_playing.Count == 0 )
                return false;

            foreach ( RunningAnim anim in m_playing ) {
                double fCurTime = m_timer.Current;
                double fDelta = fCurTime - anim.lastTime;
                anim.lastTime = fCurTime;

                bool bFinished = anim.anim.TimeDelta((float)fDelta);
                if ( bFinished )
                    m_finished.Add(anim);
            }

            // doing this because we can't remove from list while we iterate...(right?)
            foreach ( RunningAnim anim in m_finished )
                m_playing.Remove(anim);
            m_finished.Clear();

            return true;
        }


        public void Start() {
            m_timer.Start();
        }


        static public AnimationManager Get {
            get {
                if (s_singleton == null)
                    s_singleton = new AnimationManager();
                return s_singleton;
            }
        }

        protected AnimationManager() {
            m_playing = new List<RunningAnim>();
            m_finished = new List<RunningAnim>();
            m_timer = new SysTimer();
        }
        static AnimationManager s_singleton = null;
    }
}
