using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Reflection;
using Tao.OpenGl;

namespace Sketcher
{
    /*
     * [TODO] add event to Image class so that we can listen to it and update GLTexture if it changes (!)
     */
    class Icon
    {
        public enum ScaleModeType
        {
            Nearest,
            Linear
        };

        Image m_image;
        GLTexture m_texture;
        ScaleModeType m_eScaleMode;

        public Icon( Image img ) {
            m_image = img;
            m_texture = new GLTexture( m_image );
            m_texture.TexEnvMode = Gl.GL_MODULATE;
            ScaleMode = ScaleModeType.Linear;
        }

        public Image IconImage {
            get { return m_image; }
            set { m_image = value; m_texture.TexImage = m_image; }
        }

        public ScaleModeType ScaleMode {
            get { return m_eScaleMode; }
            set {
                m_eScaleMode = value;
                switch (m_eScaleMode) {
                    case ScaleModeType.Nearest:
                        m_texture.TextureFilter = Gl.GL_NEAREST;
                        break;
                    case ScaleModeType.Linear:
                        m_texture.TextureFilter = Gl.GL_LINEAR;
                        break;
                }
            }
        }
                

        public void Draw( Box2f bounds, float fAlpha ) {

            if ( m_texture != null ) 
                m_texture.Enable();

            Gl.glColor4f( 1, 1, 1, fAlpha );

            Gl.glBegin( Gl.GL_QUADS );
            Gl.glTexCoord2f( 0, 0 );
            Gl.glVertex2f( bounds.Min.X, bounds.Min.Y );
            Gl.glTexCoord2f( 1, 0 );
            Gl.glVertex2f( bounds.Max.X, bounds.Min.Y );
            Gl.glTexCoord2f( 1, 1 );
            Gl.glVertex2f( bounds.Max.X, bounds.Max.Y );
            Gl.glTexCoord2f( 0, 1 );
            Gl.glVertex2f( bounds.Min.X, bounds.Max.Y );
            Gl.glEnd();

            if (m_texture != null )
                m_texture.Disable();
        }










        static public Icon LoadIconFile( FilePath path ) {
            Image img = new Image();
            if ( img.Load(path) )
                return new Icon(img);
            else
                return null;
        }


    }
}
