using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class StrokeUtil
    {
        public static float GetCrossScore(Box2f bounds, LineStroke s, Transform sTransform) {
            Box2f strokeBounds = (sTransform == null) ? s.Bounds : sTransform.Apply(s.Bounds, true);
            if (!bounds.Intersects(strokeBounds))
                return 0.0f;

            Vector2f strokeBegin = (sTransform == null) ? s.Begin : sTransform.Apply(s.Begin, true);
            Vector2f strokeEnd = (sTransform == null) ? s.End : sTransform.Apply(s.End, true);
            Vector2f P1 = strokeBegin + s.Line * 0.25f;
            Vector2f P2 = strokeEnd - s.Line * 0.25f;

            float y1, y2;
            if (strokeBegin.Y > strokeEnd.Y) {
                y2 = P1.Y; y1 = P2.Y;
            } else {
                y1 = P1.Y; y2 = P2.Y;
            }
            float tylow = (bounds.Min.Y - y1) / (y2 - y1);
            float tyhigh = (bounds.Max.Y - y1) / (y2 - y1);
            if ((tylow < 0 && tyhigh < 0) || (tylow > 1 && tyhigh > 1))
                return 0;
            if (tyhigh > 1)
                tyhigh = 1;
            if (tylow < 0)
                tylow = 0;

            float x1, x2;
            if (strokeBegin.X > strokeEnd.X) {
                x2 = P1.X; x1 = P2.X;
            } else {
                x1 = P1.X; x2 = P2.X;
            }
            float txlow = (bounds.Min.X - x1) / (x2 - x1);
            float txhigh = (bounds.Max.X - x1) / (x2 - x1);
            if ((txlow < 0 && txhigh < 0) || (txlow > 1 && txhigh > 1))
                return 0;
            if (txhigh > 1)
                txhigh = 1;
            if (txlow < 0)
                txlow = 0;

            float tlow = Math.Max(tylow, txlow);
            float thigh = Math.Min(tyhigh, txhigh);
            if ( tlow > thigh )
                return 0;

            Vector2f vLine = new Vector2f(x2 - x1, y2 - y1);
            Vector2f pSeg = vLine * (thigh - tlow);
            float fSegLen = pSeg.Length;
            float fBoxDiag = bounds.DiagonalLen;

            float fIntersectScore = fSegLen / fBoxDiag;

            return fIntersectScore;

            // [RMS: this should be a "min-distance-to-center" test...]
            //Vector2f vIsectMidpoint = new Vector2f(x1,y1) + vLine * tlow + pSeg * 0.5f;
            //float fCenterDist = (vIsectMidpoint - bounds.Center).Length();
            //float fBoundsRad = fBoxDiag / 2.0f;
            //float fProximityScore = 1.0f - fCenterDist / fBoundsRad;
            //return 0.5f * (fIntersectScore + fProximityScore);
        }


    }
}
