using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    struct HistoryValue<T>
    {
        public DateTime timestamp;
        public T value;
        public HistoryValue(DateTime time, T val) {
            timestamp = time; value = val;
        }
    };

    class Parameter<T>
    {
        T m_value;
        Stack< HistoryValue<T> > m_history;

        public Parameter(T value) {
            m_value = value;

            m_history = new Stack<HistoryValue<T>>();
            m_history.Push( new HistoryValue<T>(DateTime.Now, m_value ) );
        }

        public T Value {
            get { return m_value; }
        }

        //! set here does not store value in history (to support preview changes)
        public void SetPreviewValue(T value) {
            m_value = value;
        }

        //! set value and store in history
        public void SetNewValue(T value) {
            m_history.Push( new HistoryValue<T>(DateTime.Now, m_value ) );
            m_value = value; 
        }

        public Stack<HistoryValue<T>> History {
            get { return m_history; }
        }
    }
}
