using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class TranslateCanvasBehavior : ItemInputBehavior
    {
        Vector2f m_captureStart;
        VectorCanvas m_canvas;

        public TranslateCanvasBehavior(Item target, VectorCanvas canvas) : base( target ) {
            m_canvas = canvas;
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }

        //! return value in [0,1] to indicate desire for capturing stroke
        override public float GetCaptureScore(Stroke s) {
            if (TargetItem.Geometry.IsInside( s.First ))
                return 1.0f;
            return 0.0f;
        }

        //! return true to continue capture
        override public bool OnCaptureStrokeStart(Stroke s) {
            m_captureStart = s.First;
            return true;
        }

        //! return true to continue capture, false to stop capture
        override public bool OnCapturedStrokeUpdate(Stroke s) {
            if (s.Vertices.Count > 1) {
                Vector2f vTranslate = s.Last - s.Vertices[s.Vertices.Count-2];
                m_canvas.TranslateChildArea( vTranslate, true );
            }
            return true;
        }
    }



    // [RMS] not implemented yet (transform does not support scaling!)

    class ScaleCanvasBehavior : WidgetInputBehavior
    {
        Vector2f m_captureStart;
        VectorCanvas m_canvas;

        public ScaleCanvasBehavior( Widget widget, VectorCanvas canvas )
            : base( widget ) {
            m_canvas = canvas;
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }

        //! return value in [0,1] to indicate desire for capturing stroke
        override public float GetCaptureScore( Stroke s ) {
            if (TargetWidget.Hit( s.First ))
                return 1.0f;
            return 0.0f;
        }

        //! return true to continue capture
        override public bool OnCaptureStrokeStart( Stroke s ) {
            m_captureStart = s.First;
            //m_initialTranslation = m_canvas.Translation;
            return true;
        }

        //! return true to continue capture, false to stop capture
        override public bool OnCapturedStrokeUpdate( Stroke s ) {
            if (s.Vertices.Count > 1) {
                //Vector2f vTranslate = s.Last - m_captureStart;
                //m_canvas.Translation = 
                //    m_initialTranslation + vTranslate;
            }
            return true;
        }
    }
}
