using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {
    class UndoSliderBehavior : ItemInputBehavior 
    {
        Notebook m_notebook;
        Vector2f m_vLast;
        float m_fSlider;

        public UndoSliderBehavior(Item button, Notebook notebook)
            : base(button) {
            m_notebook = notebook;
        }

        public override float GetCaptureScore(Stroke s) {
            if (TargetItem.Geometry.IsInside(s.First))
                return 1.0f;
            return 0.0f;
        }

        public override bool OnCaptureStrokeStart(Stroke s) {
            m_vLast = s.Last;
            m_fSlider = 0;
            return true;
        }

        public override bool OnCapturedStrokeUpdate(Stroke s) {
            const int nStepDelta = 25;
            m_fSlider += (s.Last.X - m_vLast.X);
            m_vLast = s.Last;
            while (m_fSlider > nStepDelta) {
                m_notebook.CurrentPage.Redo();
                m_fSlider -= nStepDelta;
            }
            while (m_fSlider < -nStepDelta) {
                m_notebook.CurrentPage.Undo();
                m_fSlider += nStepDelta;
            }
            return true;
        }

        public override bool OnCaptureStrokeEnd(Stroke s) {
            return (m_fSlider != 0);
        }


        //! return click "score"
        override public float GetClickScore(ClickStroke s) {
            if (TargetItem.Geometry.IsInside(s.Click))
                return 1.0f;
            return 0.0f;
        }

        override public void OnClick(ClickStroke s) {
            m_notebook.CurrentPage.Undo();
        }



    }
}
