using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class VectorCircleItem : VectorGraphicsItem
    {
        Parameter<Vector2f> m_Center;
        Parameter<float> m_Radius;

        public VectorCircleItem(Vector2f vCenter, float fRadius, ColorRGBA color) {
            ForegroundColor = color;

            m_Center = new Parameter<Vector2f>(vCenter);
            m_Radius = new Parameter<float>(fRadius);

            this.Behaviors.Add(new VectorItemTranslater(this));

            Box2f bounds = new Box2f(vCenter, fRadius);
            Resize(bounds);
        }

        public override VectorGraphicsItem Copy() {
            return new VectorCircleItem(m_Center.Value, m_Radius.Value, ForegroundColor);
        }
        override public bool Set(VectorGraphicsItem to) {
            VectorCircleItem vi = to as VectorCircleItem;
            if (vi == null)
                return false;
            ForegroundColor = vi.ForegroundColor;
            m_Center = new Parameter<Vector2f>(vi.m_Center.Value);
            m_Radius = new Parameter<float>(vi.m_Radius.Value);
            Box2f bounds = new Box2f(m_Center.Value, m_Radius.Value);
            Resize(bounds);
            return true;
        }



        //! draw object
        override public void Render() {

            VectorCanvas canvas = (VectorCanvas)Parent;
            bool bSelected = canvas.SelectedItems.IsSelected(this);

            if (bSelected) {
                ColorRGBA col = ColorRGBA.LIGHTGREY;
                foreach (HistoryValue<Vector2f> center in m_Center.History) {
                    RenderEngine.Get.DrawDisc(center.value, m_Radius.Value - 2, m_Radius.Value + 2, col);
                    col.A *= 0.75f;
                }
            }

            // HACK - this should be done by resize handler....
            m_Center.SetPreviewValue(Bounds.Center);
            m_Radius.SetPreviewValue(Bounds.Width / 2);

            RenderEngine.Get.DrawDisc(m_Center.Value, m_Radius.Value - 2, m_Radius.Value + 2, 
                (bSelected) ? ColorRGBA.RED : ForegroundColor);
        }


        /*
         * VectorGraphicsItem overrides
         */

        override public float IsHit(Vector2f vClick) {
            const float s_fThreshold = 10.0f;   // pixel
            float fDist = vClick.Dist(m_Center.Value);
            float fCircDist = (float)Math.Abs(fDist - m_Radius.Value);
            float fScore = 1 - fCircDist / s_fThreshold;
            return Math.Max(0, fScore); 
        }


        override public void NotifySizeChanged() {
            m_Center.SetNewValue(m_Center.Value);
        }
    }
}
