using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {

    class VectorStrokeTool : VectorTool 
    {
        InputStrokeRenderer m_strokeRenderer;

        public static readonly string ToolName = "StrokeTool";

        public VectorStrokeTool(VectorCanvas target) : base(target) 
        {
            m_strokeRenderer = new InputStrokeRenderer(this);
        }

        public override string Name {
            get { return ToolName; }
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }

        public override float GetCaptureScore(Stroke s) {
            if (TargetCanvas.ChildAreaBounds.Contains(s.First))
                return 0.5f;
            return 0.0f;
        }

        public override bool OnCaptureStrokeStart(Stroke s) {
            if ( m_strokeRenderer != null )
                m_strokeRenderer.StrokeBegin(s);
            return base.OnCaptureStrokeStart(s);
        }

        public override bool OnCapturedStrokeUpdate(Stroke s) {
            if (m_strokeRenderer != null)
                m_strokeRenderer.StrokeUpdate(s);
            return base.OnCapturedStrokeUpdate(s);
        }

        public override bool OnCaptureStrokeEnd(Stroke s) {
            if (m_strokeRenderer != null)
                m_strokeRenderer.StrokeEnd();
            AppendStroke(s);
            return true;
        }


        void AppendStroke(Stroke s) {
            List<Vector2f> vVertices = new List<Vector2f>(s.Vertices.Count);
            List<float> vSizes = new List<float>(s.Vertices.Count);
            List<float> vOpacties = new List<float>(s.Vertices.Count);
            for (int i = 0; i < s.Vertices.Count; ++i) {
                vVertices.Add( TargetCanvas.ToChildArea(s.Vertices[i]) );
                vSizes.Add( Stroke.PressureToSizeMap( s.Pressures[i], 0.5f, 1.0f ) );
                vOpacties.Add(Stroke.PressureToOpacityMap(s.Pressures[i], 0.1f, 0.35f));
            }

            VectorStrokeItem stroke = new VectorStrokeItem(vVertices, vSizes, vOpacties, 2.0f, ColorRGBA.BLACK);

            Box2f bounds = stroke.Bounds;
            TargetCanvas.AddItem(stroke);
        }

    }



    class InputStrokeRenderer : RenderingBehavior {
        VectorStrokeTool m_targetTool;
        Stroke m_stroke;

        public InputStrokeRenderer(VectorStrokeTool target)
            : base(target) {
            m_targetTool = target;
            m_stroke = null;
        }

        public override bool Enabled {
            get { return m_targetTool.Enabled; }
        }

        public void StrokeBegin(Stroke s) {
            m_stroke = s;
        }

        public void StrokeUpdate(Stroke s) {
            // ?
        }

        public void StrokeEnd() {
            m_stroke = null;
        }


        //! this draws the whole stroke into the overlay, so no updates are needed. It is
        //  faster for short strokes, but for long ones the drawing time starts to bog down the system...
        public override bool OnOverlayRender() {
            if (m_stroke == null || m_stroke.Vertices.Count < 2)
                return false;
            for (int i = 0; i < m_stroke.Vertices.Count-1; ++i) {
                RenderEngine.Get.DrawLine(
                    m_stroke.Vertices[i], m_stroke.Vertices[i+1],
                    Stroke.PressureToSizeMap( m_stroke.Pressures[i], 0.5f, 1.0f ),
                    Stroke.PressureToSizeMap( m_stroke.Pressures[i+1], 0.5f, 1.0f ),
                    Stroke.PressureToOpacityMap( m_stroke.Pressures[i], 0.1f, 0.35f ),
                    Stroke.PressureToOpacityMap( m_stroke.Pressures[i+1], 0.1f, 0.35f ),
                    2.0f, ColorRGBA.BLACK );
            }

            return true;
        }
        public override bool OverlayRenderFlag {
//            get { return false; }
            get { return (m_stroke != null); }
        }


    }


}
