using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {

    abstract class VectorTool : ItemInputBehavior 
    {
        Vector2f m_vLastInputPosition;
        Widget m_cursor;


        public VectorTool(VectorCanvas target) : base(target) 
        {
            m_vLastInputPosition = Vector2f.ZERO;
            m_cursor = null;
        }

        abstract public string Name {
            get;
        }

        public VectorCanvas TargetCanvas {
            get { return Target as VectorCanvas; }
        }

        public Vector2f LastInputPosition {
            get { return m_vLastInputPosition; }
        }

        public Widget Cursor {
            get { return m_cursor; }
            set { m_cursor = value; }
        }


        public override bool Enabled {
            get { return base.Enabled && TargetCanvas.Tools.SelectedTool == this; }
        }


        override public void OnHover(Vector2f v) {
            m_vLastInputPosition = v;

            if (m_cursor != null) {
                Box2f newPos = m_cursor.Bounds;
                newPos.Center = LastInputPosition;
                m_cursor.Resize(newPos);
            }
        }

        public override bool OnCapturedStrokeUpdate(Stroke s) {
            m_vLastInputPosition = s.Last;

            if (m_cursor != null) {
                Box2f newPos = m_cursor.Bounds;
                newPos.Center = LastInputPosition;
                m_cursor.Resize(newPos);
            }

            return base.OnCapturedStrokeUpdate(s);
        }

        virtual public void Enable() {
            Connect();
            if (m_cursor != null) {
                WidgetManager.Get.Register(m_cursor);
                m_cursor.Visibility = VisibilityState.Visible;  // no transition
                //m_cursor.Show();
            }
        }

        virtual public void Disable() {
            Disconnect();
            if (m_cursor != null) {
                WidgetManager.Get.DeRegister(m_cursor);
                m_cursor.Visibility = VisibilityState.Hidden;  // no transition
                //m_cursor.Hide();
            }
        }


    }
}
