using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class RadialMenu : Widget
    {
        Circle2f m_circle;

        public RadialMenu( Element parent, float fRadius ) : base( parent ) {
            Resize( new Box2f( 2*fRadius, 2*fRadius ) );
            m_circle = new Circle2f(Vector2f.ZERO, fRadius);

            BackgroundColor = ColorRGBA.DARKDARKGREY;
            ForegroundColor = ColorRGBA.WHITE;

            // by default, add behavior that prevents capture if input stroke hits this object
            this.Behaviors.Add(new PreventCaptureBehavior(this));
            this.Behaviors.Add(new FadeInOutBehavior(this));
        }

        public Vector2f Center {
            get { return Bounds.Center; }
            set {
                Box2f tmp = Bounds;
                tmp.Translate(value - Bounds.Center);
                Resize(tmp); 
            }
        }

        /*
         * Widget Overrides
         */
        //! clipping bounds for widget
        override public Box2f ClipRegion {
            get { return Parent.Parent.Bounds; }
        }

        override protected void UpdateGeometry() {
            m_circle.Center = Bounds.Center;
        }

        override protected void RenderWidget() {
            m_circle.Center = Bounds.Center;

            RenderEngine.Get.DrawCircle(m_circle, AlphaBackgroundColor);
            RenderEngine.Get.DrawCircle(m_circle.Center, m_circle.Radius-2.0f, AlphaForegroundColor);
        }


    }
}
